local ipairs, wipe, CreateFrame, insert, type = ipairs, table.wipe, CreateFrame, table.insert, type;

local function Resizing(ScrollFrame)
    local WidthAltered = false
    if ScrollFrame.Content:GetHeight() > (ScrollFrame:GetHeight() - 2*ScrollFrame.self.BorderSize) then 
        if not ScrollFrame.DownBtn then 
            ScrollFrame.DownBtn = ScrollFrame.self:CreateButton(ScrollFrame)
            ScrollFrame.DownBtn:SetPoint("BOTTOMRIGHT")
            ScrollFrame.DownBtn:Size(20, 20)
            --ScrollFrame.DownBtn:SetIcon("Interface\\BUTTONS\\UI-MicroStream-Yellow.blp", 0.2, 0.8, 0.2, 0.8, "Highlight")
            ScrollFrame.DownBtn:SetIcon("TexturePath",ScrollFrame.self:GetIcon("ArrowHeadMask"),"Theme","Highlight","Rotation",-45,"Zoom",0)
            --ScrollFrame.DownBtn.Icon:SetTexCoord(1.2,0.5,0.5,1.2,0.5,-0.2,-0.2,0.5)
            ScrollFrame.DownBtn.OnClick = function()
				local ScrollMulti = ScrollFrame.Content:GetHeight()/10
				if ScrollMulti > ScrollFrame:GetHeight() - 2 * ScrollFrame.self.BorderSize then 
					ScrollMulti = ScrollFrame:GetHeight() - 2 * ScrollFrame.self.BorderSize
				end
                ScrollFrame:Refresh(ScrollMulti, true)
            end
            WidthAltered = true 
        end
    
        if not ScrollFrame.UpBtn then 
            ScrollFrame.UpBtn = ScrollFrame.self:CreateButton(ScrollFrame)
            ScrollFrame.UpBtn:SetPoint("TOPRIGHT", 0, ScrollFrame.TopOffset)
            ScrollFrame.UpBtn:Size(20, 20)
            --ScrollFrame.UpBtn:SetIcon("Interface\\BUTTONS\\UI-MicroStream-Yellow.blp", 0.8, 0.2, 0.8, 0.2, "Highlight")
            ScrollFrame.UpBtn:SetIcon("TexturePath",ScrollFrame.self:GetIcon("ArrowHeadMask"),"Theme","Highlight","Rotation",135,"Zoom",0)
            --ScrollFrame.UpBtn.Icon:SetTexCoord(-0.2,0.5,0.5,-0.2,0.5, 1.2,1.2,0.5)
            ScrollFrame.UpBtn.OnClick = function()
				local ScrollMulti = ScrollFrame.Content:GetHeight()/10
				if ScrollMulti > ScrollFrame:GetHeight() - 2 * ScrollFrame.self.BorderSize then 
					ScrollMulti = ScrollFrame:GetHeight() - 2 * ScrollFrame.self.BorderSize
				end
                ScrollFrame:Refresh(-ScrollMulti, true)
            end
        end
    
        if not ScrollFrame.SlidLine then 
            ScrollFrame.SlidLine = ScrollFrame.self:FetchTexture(ScrollFrame)
            ScrollFrame.SlidLine:SetPoint("TOP", ScrollFrame.UpBtn, "BOTTOM")
            ScrollFrame.SlidLine:SetPoint("BOTTOM", ScrollFrame.DownBtn, "TOP")
            ScrollFrame.SlidLine:SetWidth(2)
            ScrollFrame.SlidLine:SetColorTexture(1,1,1,1)
            ScrollFrame.self:SetThemedColor(ScrollFrame.SlidLine,"Highlight")
        end
    
        if not ScrollFrame.SliderButton then 
            ScrollFrame.SliderButton = ScrollFrame.self:CreateFrame(ScrollFrame)
            ScrollFrame.SliderButton:Size(20, 20)
            ScrollFrame.SliderButton:SetPoint("TOP", ScrollFrame.SlidLine, "TOP", 0, -ScrollFrame.self.Padding)
            ScrollFrame.SliderButton:SetThemedBackdrop()
            ScrollFrame.SliderButton:SetThemedBackdrop("Backdrop", "Highlight")
        end
        
        ScrollFrame.ScrollFrameBackdrop:SetPoint("TOPRIGHT", ScrollFrame.UpBtn, "TOPLEFT", -ScrollFrame.self.Padding, 0)
        ScrollFrame.Content:SetWidth(ScrollFrame:GetWidth() - 2 * ScrollFrame.self.BorderSize - ScrollFrame.self.Padding - ScrollFrame.UpBtn:GetWidth())
    else 
        if ScrollFrame.DownBtn then 
            ScrollFrame.DownBtn:Release()
            ScrollFrame.DownBtn = nil
            WidthAltered = true 
        end 
        if ScrollFrame.UpBtn then 
            ScrollFrame.UpBtn:Release()
            ScrollFrame.UpBtn = nil
        end 
        if ScrollFrame.SlidLine then 
            ScrollFrame.SlidLine:Release()
            ScrollFrame.SlidLine = nil
        end 
        if ScrollFrame.SliderButton then 
            ScrollFrame.SliderButton:Release()
            ScrollFrame.SliderButton = nil
        end 
        ScrollFrame.ScrollFrameBackdrop:SetPoint("TOPRIGHT")
        ScrollFrame.Content:SetWidth(ScrollFrame:GetWidth() - 2 * ScrollFrame.self.BorderSize)  
    end 
       
    if WidthAltered and ScrollFrame.OnContentWidthChanged then 
        ScrollFrame:OnContentWidthChanged()
    end   
    ScrollFrame:Refresh()
	ScrollFrame:SetScript("OnUpdate",nil)
end 

function _Avyr_Global_ADDON_Elements_Lib.WidgetsMixin:CreateAdancedScrollFrame(Parent)
    local ScrollFrame = self:CreateFrame(Parent)

    ScrollFrame.ScrollFrame = self:CreateScrollFrame(ScrollFrame)

    ScrollFrame.Content = self:CreateFrame(ScrollFrame.ScrollFrame)
    ScrollFrame.Content:SetPoint("TOPLEFT")
    ScrollFrame.Content:Size(600, 100)
    ScrollFrame.Content:SetClampedToScreen(false)
    ScrollFrame.ScrollFrame:SetScrollChild(ScrollFrame.Content)

    ScrollFrame:SetScript("OnMouseWheel",
    function(_, delta)
		local ScrollMulti = ScrollFrame.Content:GetHeight()/10
		if ScrollMulti > ScrollFrame:GetHeight() - 2 * ScrollFrame.self.BorderSize then 
			ScrollMulti = ScrollFrame:GetHeight() - 2 * ScrollFrame.self.BorderSize
		end
        ScrollFrame:Refresh(-ScrollMulti * delta, true)
    end)

	ScrollFrame:SetScript("OnMouseDown",function()
		if ScrollFrame.UpBtn then 
			local MouseX, MouseY = GetCursorPosition()
			local Scale = ScrollFrame:GetEffectiveScale() 
			MouseX = MouseX / Scale
			MouseY = MouseY / Scale
			local TopLimit = ScrollFrame.UpBtn:GetBottom() - ScrollFrame.self.Padding
			local BottomLimit = ScrollFrame.DownBtn:GetTop() + ScrollFrame.self.Padding
			if ScrollFrame:GetRight() > MouseX and ScrollFrame.ScrollFrame:GetRight() < MouseX and MouseY < TopLimit and MouseY > BottomLimit then 
				local Length = TopLimit - BottomLimit
				local ContentHeightRef = ScrollFrame.Content:GetHeight()
				local SliderTop = ScrollFrame.SliderButton:GetTop()
				local SliderBottom = ScrollFrame.SliderButton:GetBottom()
				local SliderOffset
				if MouseY > SliderBottom and MouseY < SliderTop then 
					SliderOffset = SliderTop - MouseY 
				else
					SliderOffset = ScrollFrame.SliderButton:GetHeight() / 2
				end
				ScrollFrame:SetScript("OnUpdate",function()
					local NewMouseX, NewMouseY = GetCursorPosition()
					--NewMouseX = NewMouseX / Scale
					NewMouseY = NewMouseY / Scale - BottomLimit + SliderOffset
					ScrollFrame:Refresh(ContentHeightRef * (1 - NewMouseY / Length))
				end)
			end
		end
	end)

	ScrollFrame:SetScript("OnMouseUp",function()
		ScrollFrame:SetScript("OnUpdate",nil)
	end)

    ScrollFrame.ScrollFrameBackdrop = self:CreateFrame(ScrollFrame)
    ScrollFrame.ScrollFrameBackdrop:SetThemedBackdrop()
    ScrollFrame.ScrollFrameBackdrop:SetPoint("BOTTOMLEFT")
    ScrollFrame.ScrollFrameBackdrop:SetPoint("TOPRIGHT")

    ScrollFrame.ScrollFrame:SetPoint("BOTTOMLEFT", ScrollFrame.ScrollFrameBackdrop, self.BorderSize, self.BorderSize)
    ScrollFrame.ScrollFrame:SetPoint("TOPRIGHT", ScrollFrame.ScrollFrameBackdrop, -self.BorderSize, -self.BorderSize)

    ScrollFrame.Refresh = function(selfScrollFrame, ScrollOffset, Increment)
        ScrollOffset = ScrollOffset or selfScrollFrame.ScrollOffset
        if Increment then 
            ScrollOffset = ScrollOffset + selfScrollFrame.ScrollOffset
        else 
            ScrollOffset = ScrollOffset
        end 

        local ContentHeight = selfScrollFrame.Content:GetHeight()
        if ScrollOffset < 0 then 
            ScrollOffset = 0
        end 
        if ContentHeight < selfScrollFrame:GetHeight() - 2*self.BorderSize then 
            ScrollOffset = 0
        elseif ScrollOffset > ContentHeight - (selfScrollFrame:GetHeight() - 2*self.BorderSize) then 
            ScrollOffset = ContentHeight - (selfScrollFrame:GetHeight() - 2*self.BorderSize)
        end 

        selfScrollFrame.ScrollFrame:SetVerticalScroll(ScrollOffset)

        if selfScrollFrame.UpBtn then 
            local SliderHeight = selfScrollFrame:GetHeight() - 2*self.Padding - selfScrollFrame.UpBtn:GetHeight() - selfScrollFrame.DownBtn:GetHeight()
            local SliderHeightMulti = (selfScrollFrame:GetHeight() - 2*self.BorderSize) / ContentHeight
            if SliderHeightMulti > 1 then 
                SliderHeightMulti = 1
            end 

            selfScrollFrame.SliderButton:SetPoint("TOP", ScrollFrame.SlidLine, "TOP", 0, -self.Padding - (SliderHeight * (ScrollOffset/selfScrollFrame.Content:GetHeight())))
            selfScrollFrame.SliderButton:Size(nil, SliderHeightMulti * SliderHeight)
        end 

        if ScrollOffset ~= selfScrollFrame.ScrollOffset then 
            if selfScrollFrame.OnScroll then 
                selfScrollFrame.OnScroll(selfScrollFrame, ScrollOffset, selfScrollFrame.ScrollOffset)
            end 
        end 
        selfScrollFrame.ScrollOffset = ScrollOffset
    end 

    ScrollFrame:SizeChanged(function()
        --ScrollFrame.Content:Size(ScrollFrame:GetWidth() - 2 * self.BorderSize - self.Padding - ScrollFrame.UpBtn:GetWidth(), nil)
        Resizing(ScrollFrame)
    end)

    ScrollFrame.Content:SizeChanged(function()
        --ScrollFrame.Content:SetWidth(ScrollFrame:GetWidth() - 2 * self.BorderSize - self.Padding - ScrollFrame.UpBtn:GetWidth())
        Resizing(ScrollFrame)
    end)

    ScrollFrame.ScrollOffset = 0;

    return ScrollFrame
end 